using System;
using System.IO;
using System.Linq;
using System.Windows;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Spreadsheet;
using Microsoft.Win32;
using SMT.MastaAPI.Gears;
using SMT.MastaAPI.Gears.GearDesigns.Cylindrical;
using SMT.MastaAPI.SystemModel;
using SMT.MastaAPI.SystemModel.PartModel;
using SMT.MastaAPIUtility.Scripting;
using SMT.MastaAPIUtility.UnitsAndMeasurements;

namespace ExcelImportExample
{
    public static class AssemblyExtensions
    {
        [MastaProperty(name: "Export Geometry to Excel", description: "", symbol: "", measurement: MeasurementType.Unmeasurable)]
        public static ScriptingObjectCommand<CylindricalGearSetDesign> ExportGeometrytoExcel(this CylindricalGearSetDesign cylindricalGearSetDesign)
        {
            var command = new ScriptingObjectCommand<CylindricalGearSetDesign>(exportGeometrytoExcelCode, cylindricalGearSetDesign);
            return command;
        }

        private static void exportGeometrytoExcelCode(CylindricalGearSetDesign cylindricalGearSetDesign)
        {
            var openFileDialog = new OpenFileDialog();
            if (openFileDialog.ShowDialog() != true)
                return;

            var openSettings = new OpenSettings() { AutoSave = false };
            using (var fs = File.Open(openFileDialog.FileName, FileMode.OpenOrCreate, FileAccess.ReadWrite))
            using (var spreadsheet = SpreadsheetDocument.Open(fs, isEditable: true, openSettings: openSettings))
            {
                var sheet = spreadsheet.WorkbookPart.WorksheetParts.First().Worksheet;
                var sheetData = sheet.GetFirstChild<SheetData>();
                var row = sheetData.Elements<Row>().FirstOrDefault();
                if ( row == null )
                {
                    row = new Row() { RowIndex = 1 };
                    sheetData.AppendChild(row);
                }
                setCellData(row, 0, cylindricalGearSetDesign.CylindricalMeshes[0].CentreDistance);
                setCellData(row, 1, cylindricalGearSetDesign.HelixAngle);
                setCellData(row, 2, cylindricalGearSetDesign.NormalPressureAngle);
                setCellData(row, 3, cylindricalGearSetDesign.NormalModule);
                setCellData(row, 4, cylindricalGearSetDesign.CylindricalGears[0].FaceWidth);
                setCellData(row, 5, cylindricalGearSetDesign.CylindricalGears[1].FaceWidth);
                setCellData(row, 6, cylindricalGearSetDesign.CylindricalGears[0].Hand);
                setCellData(row, 7, cylindricalGearSetDesign.CylindricalGears[0].NumberOfTeeth);
                setCellData(row, 8, cylindricalGearSetDesign.CylindricalGears[1].NumberOfTeeth);

                sheet.Save();
            }
        }

        [MastaProperty(name: "Import Geometry from Excel", description: "", symbol: "", measurement: MeasurementType.Unmeasurable)]
        public static ScriptingObjectCommand<Assembly> ImportGeometryFromExcel(this Assembly assembly)
        {
            var command = new ScriptingObjectCommand<Assembly>(importGeometryFromExcelCode, assembly);
            return command;
        }

        private static void importGeometryFromExcelCode(Assembly assembly)
        {
            var openFileDialog = new OpenFileDialog();
            if (openFileDialog.ShowDialog() != true)
                return;
            var openSettings = new OpenSettings() { AutoSave = false };
            using (var fs = File.Open(openFileDialog.FileName, FileMode.Open, FileAccess.Read))
            using (var spreadsheet = SpreadsheetDocument.Open(fs, isEditable: false, openSettings: openSettings))
            {
                var sheet = spreadsheet.WorkbookPart.WorksheetParts.First().Worksheet;
                var sheetData = sheet.GetFirstChild<SheetData>();
                var row = sheetData.Elements<Row>().First();
                var centreDistance = cellData(row, 0);
                var helixAngle = cellData(row, 1);
                var normalPressureAngle = cellData(row, 2);
                var normalModule = cellData(row, 3);
                var gearAFaceWidth = cellData(row, 4);
                var gearBFaceWidth = cellData(row, 5);
                var pinionHand = (Hand)Enum.Parse(typeof(Hand), row.Elements<Cell>().ElementAt(6).InnerText);
                var gearANumberOfTeeth = cellData(row, 7);
                var gearBNumberOfTeeth = cellData(row, 8);
                var cylindricalGearSet = assembly.AddCylindricalGearSet("Imported Gear Set", normalPressureAngle, helixAngle, normalModule, pinionHand, centreDistance);
                var cylindricalGearSetDesign = cylindricalGearSet.CylindricalGearSetDesign;
                cylindricalGearSetDesign.CylindricalGears[0].FaceWidth = gearAFaceWidth;
                cylindricalGearSetDesign.CylindricalGears[1].FaceWidth = gearBFaceWidth;
                cylindricalGearSetDesign.CylindricalGears[0].NumberOfTeeth = (int)gearANumberOfTeeth;
                cylindricalGearSetDesign.CylindricalGears[1].NumberOfTeeth = (int)gearBNumberOfTeeth;
            }
        }

        private static void setCellData(Row row, int index, object value)
        {
            var cell = row.Elements<Cell>().ElementAtOrDefault(index);
            if ( cell == null )
            {
                cell = new Cell();
                row.InsertAt(cell, index);
            }
            var valueAsString = value.ToString();
            cell.CellValue = new CellValue(valueAsString);

            double temp;
            var cellValueType = double.TryParse(valueAsString, out temp)
                ? CellValues.Number
                : CellValues.String;

            cell.DataType = new EnumValue<CellValues>(cellValueType);
        }

        private static double cellData(Row row, int index)
        {
            var cell = row.Elements<Cell>().ElementAt(index);
            return double.Parse(cell.CellValue.Text);
        }
    }
}